@@ -118,9 +118,9 @@ public:
 	//These are public, but not slots, because they don't use sufficient validation. Scripts shouldn't be able to add/remove telescopes, only to point them.
 	//! Adds a telescope description containing the given properties. DOES NOT VALIDATE its parameters. If serverName is specified, portSerial should be specified too. Call saveTelescopes() to write the modified configuration to disc. Call startTelescopeAtSlot() to start this telescope.
 	//! @param portSerial must be a valid serial port name for the particular platform, e.g. "COM1" for Microsoft Windows of "/dev/ttyS0" for Linux
-	bool addTelescopeAtSlot(int slot, ConnectionType connectionType, QString name, QString host = QString("localhost"), int portTCP = DEFAULT_TCP_PORT, int delay = DEFAULT_DELAY, bool connectAtStartup = false, QList<double> circles = QList<double>(), QString serverName = QString(), QString portSerial = QString());
+	bool addTelescopeAtSlot(int slot, ConnectionType connectionType, QString name, QString equinox, QString host = QString("localhost"), int portTCP = DEFAULT_TCP_PORT, int delay = DEFAULT_DELAY, bool connectAtStartup = false, QList<double> circles = QList<double>(), QString serverName = QString(), QString portSerial = QString());
 	//! Retrieves a telescope description. Returns false if the slot is empty. Returns empty serverName and portSerial if the description contains no server.
-	bool getTelescopeAtSlot(int slot, ConnectionType& connectionType, QString& name, QString& host, int& portTCP, int& delay, bool& connectAtStartup, QList<double>& circles, QString& serverName, QString& portSerial);
+	bool getTelescopeAtSlot(int slot, ConnectionType& connectionType, QString& name, QString& equinox, QString& host, int& portTCP, int& delay, bool& connectAtStartup, QList<double>& circles, QString& serverName, QString& portSerial);
 	//! Removes info from the tree. Should it include stopTelescopeAtSlot()?
 	bool removeTelescopeAtSlot(int slot);
 	
@@ -139,10 +139,6 @@ public:
 	//! Returns a list of the currently connected clients
 	QHash<int, QString> getConnectedClientsNames();
 	
-	//! Returns the module-specific style sheet.
-	//! The main StelStyle instance should be passed.
-	const StelStyle getModuleStyleSheet(const StelStyle& style);
-	
 	bool getFlagUseServerExecutables() {return useServerExecutables;}
 	//! Forces a call of loadDeviceModels(). Stops all active telescopes.
 	void setFlagUseServerExecutables(bool b);
@@ -268,10 +264,6 @@ private:
 	TelescopeDialog * telescopeDialog;
 	SlewDialog * slewDialog;
 	
-	//Styles
-	QByteArray * normalStyleSheet;
-	QByteArray * nightStyleSheet;
-	
 	//! Used internally. Checks if the argument is a valid slot number.
 	bool isValidSlotNumber(int slot);
 	bool isValidPort(uint port);
@@ -286,7 +278,7 @@ private:
 	bool stopServerAtSlot(int slot);
 
 	//! A wrapper for TelescopeClient::create(). Used internally by loadTelescopes() and startTelescopeAtSlot(). Does not perform any validation on its arguments.
-	bool startClientAtSlot(int slot, ConnectionType connectionType, QString name, QString host, int portTCP, int delay, QList<double> circles, QString serverName = QString(), QString portSerial = QString());
+	bool startClientAtSlot(int slot, ConnectionType connectionType, QString name, QString equinox, QString host, int portTCP, int delay, QList<double> circles, QString serverName = QString(), QString portSerial = QString());
 	
 	//! Returns true if the TelescopeClient at this slot has been stopped successfully or doesn't exist
 	bool stopClientAtSlot(int slot);
