@@ -34,11 +34,18 @@
 #include <QObject>
 
 #include "StelApp.hpp"
+#include "StelCore.hpp" //Needed for getting StelNavigator instances
 #include "StelObject.hpp"
 #include "StelNavigator.hpp"
+#include "InterpolatedPosition.hpp"
 
 qint64 getNow(void);
 
+enum Equinox {
+	EquinoxJ2000,
+	EquinoxJNow
+};
+
 //! An abstract base class that should never be used directly, only inherited.
 //! This class used to be called Telescope, but it has been renamed
 //! to TelescopeClient in order to resolve a compiler/linker conflict
@@ -139,16 +146,6 @@ private:
 	Vec3d desired_pos;
 };
 
-//! A telescope's position at a given time.
-//! This structure used to be defined inline in TelescopeTCP.
-struct Position
-{
-	qint64 server_micros;
-	qint64 client_micros;
-	Vec3d pos;
-	int status;
-};
-
 //! This TelescopeClient class can controll a telescope by communicating
 //! to a server process ("telescope server") via 
 //! the "Stellarium telescope control protocol" over TCP/IP.
@@ -158,7 +155,7 @@ class TelescopeTCP : public TelescopeClient
 {
 	Q_OBJECT
 public:
-	TelescopeTCP(const QString &name,const QString &params);
+	TelescopeTCP(const QString &name, const QString &params, Equinox eq = EquinoxJ2000);
 	~TelescopeTCP(void)
 	{
 		hangup();
@@ -183,7 +180,6 @@ private:
 	
 private:
 	void hangup(void);
-	void resetPositions(void);
 	QHostAddress address;
 	unsigned int port;
 	QTcpSocket * tcpSocket;
@@ -195,13 +191,13 @@ private:
 	char *writeBufferEnd;
 	int time_delay;
 
-	Position positions[16];
-	Position *position_pointer;
-	Position *const end_position;
+	InterpolatedPosition interpolatedPosition;
 	virtual bool hasKnownPosition(void) const
 	{
-		return (position_pointer->client_micros != 0x7FFFFFFFFFFFFFFFLL);
+		return interpolatedPosition.isKnown();
 	}
+
+	Equinox equinox;
 	
 private slots:
 	void socketFailed(QAbstractSocket::SocketError socketError);
