@@ -33,12 +33,11 @@
 #include <QRegExp>
 #include <QStringList>
 
-TelescopeClientDirectLx200::TelescopeClientDirectLx200
-                            (const QString &name, const QString &parameters) : 
-                             TelescopeClient(name),
-                             end_position(positions + (sizeof(positions)/sizeof(positions[0])))
+TelescopeClientDirectLx200::TelescopeClientDirectLx200 (const QString &name, const QString &parameters, Equinox eq) :
+		TelescopeClient(name),
+		equinox(eq)
 {
-	resetPositions();
+	interpolatedPosition.reset();
 	
 	//Extract parameters
 	//Format: "serial_port_name:time_delay"
@@ -69,9 +68,9 @@ TelescopeClientDirectLx200::TelescopeClientDirectLx200
 	
 	#ifdef Q_OS_WIN32
 	if(serialDeviceName.right(serialDeviceName.size() - 3).toInt() > 9)
-		serialDeviceName = "\\\\.\\" + serialDeviceName + ":";//"\\.\COMxx", not sure if it will work
+		serialDeviceName = "\\\\.\\" + serialDeviceName;//"\\.\COMxx", not sure if it will work
 	else
-		serialDeviceName = serialDeviceName + ":";
+		serialDeviceName = serialDeviceName;
 	#endif //Q_OS_WIN32
 	
 	//Try to establish a connection to the telescope
@@ -92,46 +91,37 @@ TelescopeClientDirectLx200::TelescopeClientDirectLx200
 	answers_received = false;
 }
 
-//! resets/initializes the array of positions kept for position interpolation
-void TelescopeClientDirectLx200::resetPositions()
-{
-	for (position_pointer = positions; position_pointer < end_position; position_pointer++)
-	{
-		position_pointer->server_micros = 0x7FFFFFFFFFFFFFFFLL;
-		position_pointer->client_micros = 0x7FFFFFFFFFFFFFFFLL;
-		position_pointer->pos[0] = 0.0;
-		position_pointer->pos[1] = 0.0;
-		position_pointer->pos[2] = 0.0;
-		position_pointer->status = 0;
-	}
-	position_pointer = positions;
-}
-
-//! queues a GOTO command with the specified position to the write buffer.
-//! For the data format of the command see the
-//! "Stellarium telescope control protocol" text file
+//! queues a GOTO command
 void TelescopeClientDirectLx200::telescopeGoto(const Vec3d &j2000Pos)
 {
-	if (isConnected())//TODO: See the else clause, think how to do the same thing
+	if (!isConnected())
+		return;
+
+	Vec3d position = j2000Pos;
+	if (equinox == EquinoxJNow)
 	{
-		//if (writeBufferEnd - writeBuffer + 20 < (int)sizeof(writeBuffer))
-		{
-			const double ra_signed = atan2(j2000Pos[1], j2000Pos[0]);
-			//Workaround for the discrepancy in precision between Windows/Linux/PPC Macs and Intel Macs:
-			const double ra = (ra_signed >= 0) ? ra_signed : (ra_signed + 2.0 * M_PI);
-			const double dec = atan2(j2000Pos[2], sqrt(j2000Pos[0]*j2000Pos[0]+j2000Pos[1]*j2000Pos[1]));
-			unsigned int ra_int = (unsigned int)floor(0.5 + ra*(((unsigned int)0x80000000)/M_PI));
-			int dec_int = (int)floor(0.5 + dec*(((unsigned int)0x80000000)/M_PI));
-			
-			gotoReceived(ra_int, dec_int);
-		}
-		/*
+		const StelNavigator* navigator = StelApp::getInstance().getCore()->getNavigator();
+		position = navigator->j2000ToEquinoxEqu(j2000Pos);
+	}
+
+	//if (writeBufferEnd - writeBuffer + 20 < (int)sizeof(writeBuffer))
+	//TODO: See the else clause, think how to do the same thing
+	{
+		const double ra_signed = atan2(position[1], position[0]);
+		//Workaround for the discrepancy in precision between Windows/Linux/PPC Macs and Intel Macs:
+		const double ra = (ra_signed >= 0) ? ra_signed : (ra_signed + 2.0 * M_PI);
+		const double dec = atan2(position[2], sqrt(position[0]*position[0]+position[1]*position[1]));
+		unsigned int ra_int = (unsigned int)floor(0.5 + ra*(((unsigned int)0x80000000)/M_PI));
+		int dec_int = (int)floor(0.5 + dec*(((unsigned int)0x80000000)/M_PI));
+
+		gotoReceived(ra_int, dec_int);
+	}
+	/*
 		else
 		{
 			qDebug() << "TelescopeTCP(" << name << ")::telescopeGoto: "<< "communication is too slow, I will ignore this command";
 		}
-		*/
-	}
+	*/
 }
 
 void TelescopeClientDirectLx200::gotoReceived(unsigned int ra_int, int dec_int)
@@ -143,35 +133,8 @@ void TelescopeClientDirectLx200::gotoReceived(unsigned int ra_int, int dec_int)
 //! telescope positions:
 Vec3d TelescopeClientDirectLx200::getJ2000EquatorialPos(const StelNavigator*) const
 {
-	if (position_pointer->client_micros == 0x7FFFFFFFFFFFFFFFLL)
-	{
-		return Vec3d(0,0,0);
-	}
 	const qint64 now = getNow() - time_delay;
-	const Position *p = position_pointer;
-	do
-	{
-		const Position *pp = p;
-		if (pp == positions) pp = end_position;
-		pp--;
-		if (pp->client_micros == 0x7FFFFFFFFFFFFFFFLL) break;
-		if (pp->client_micros <= now && now <= p->client_micros)
-		{
-			if (pp->client_micros != p->client_micros)
-			{
-				Vec3d rval = p->pos * (now - pp->client_micros) + pp->pos * (p->client_micros - now);
-				double f = rval.lengthSquared();
-				if (f > 0.0)
-				{
-					return (1.0/sqrt(f))*rval;
-				}
-			}
-			break;
-		}
-		p = pp;
-	}
-	while (p != position_pointer);
-	return p->pos;
+	return interpolatedPosition.get(now);
 }
 
 bool TelescopeClientDirectLx200::prepareCommunication()
@@ -264,19 +227,15 @@ void TelescopeClientDirectLx200::sendPosition(unsigned int ra_int, int dec_int,
 {
 	//Server time is "now", because this class is the server
 	const qint64 server_micros = (qint64) getNow();
-	
-	// remember the time and received position so that later we
-	// will know where the telescope is pointing to:
-	position_pointer++;
-	if (position_pointer >= end_position)
-		position_pointer = positions;
-	position_pointer->server_micros = server_micros;
-	position_pointer->client_micros = getNow();
 	const double ra  =  ra_int * (M_PI/(unsigned int)0x80000000);
 	const double dec = dec_int * (M_PI/(unsigned int)0x80000000);
 	const double cdec = cos(dec);
-	position_pointer->pos[0] = cos(ra)*cdec;
-	position_pointer->pos[1] = sin(ra)*cdec;
-	position_pointer->pos[2] = sin(dec);
-	position_pointer->status = status;
+	Vec3d position(cos(ra)*cdec, sin(ra)*cdec, sin(dec));
+	Vec3d j2000Position = position;
+	if (equinox == EquinoxJNow)
+	{
+		const StelNavigator* navigator = StelApp::getInstance().getCore()->getNavigator();
+		j2000Position = navigator->equinoxEquToJ2000(position);
+	}
+	interpolatedPosition.add(j2000Position, getNow(), server_micros, status);
 }
