@@ -155,7 +155,7 @@ StelMainGraphicsView::StelMainGraphicsView(QWidget* parent)
 	  flagInvertScreenShotColors(false),
 	  screenShotPrefix("stellarium-"),
 	  screenShotDir(""),
-	  cursorTimeout(-1.f), flagCursorTimeout(false), minFpsTimer(NULL), maxfps(10000.f)
+	  cursorTimeout(-1.f), flagCursorTimeout(false), minFpsTimer(NULL), maxfps(10000.f), doPaint(true)
 {
 	StelApp::initStatic();
 
@@ -289,7 +289,10 @@ void StelMainGraphicsView::thereWasAnEvent()
 void StelMainGraphicsView::drawBackground(QPainter*, const QRectF&)
 {
 	const double now = StelApp::getTotalRunTime();
-
+	if(!doPaint)
+	{
+	  return;
+	}
 	// Determines when the next display will need to be triggered
 	// The current policy is that after an event, the FPS is maximum for 2.5 seconds
 	// after that, it switches back to the default minfps value to save power
@@ -313,6 +316,23 @@ void StelMainGraphicsView::drawBackground(QPainter*, const QRectF&)
 	}
 }
 
+void StelMainGraphicsView::setDoPaint(bool doPainting)
+{
+        doPaint = doPainting;
+	mainSkyItem->setDoPaint(doPainting);
+	if (minFpsTimer==NULL)
+	  return;
+
+        if (!doPaint)
+        {
+                minFpsTimer->stop();
+        }
+        else
+        {
+                minFpsTimer->start();
+        }
+}
+
 void StelMainGraphicsView::startMainLoop()
 {
 	// Set a timer refreshing for every minfps frames
