@@ -58,6 +58,7 @@ public:
 	//! Use its layout if you want to add widget on the side of the main sky view.
 	QGraphicsWidget* getTopLevelGraphicsWidget() {return backItem;}
 
+        void setDoPaint(bool doPainting);
 #ifndef DISABLE_SCRIPTING
 	//! Get the script API proxy (for signal handling)
 	StelMainScriptAPIProxy* getMainScriptAPIProxy() {return scriptAPIProxy;}
@@ -107,11 +108,12 @@ public slots:
 	//! Updates the scene and process all events
 	void updateScene() {
 
-		//#ifdef QT_MAC_USE_COCOA
-			// This call solves the problems with the qt event dispatche. The stack grew huge and many events were discarded
-			// http://bugreports.qt.nokia.com/browse/QTBUG-7502
-		//	QCoreApplication::processEvents(QEventLoop::AllEvents);
-		//#endif
+		#ifdef QT_MAC_USE_COCOA
+			 //This call solves the problems with the qt event dispatcher. The stack grew huge and many events were discarded
+			 //http://bugreports.qt.nokia.com/browse/QTBUG-7502
+			 //The previous bug has been closed an fixed but this line is still needed. The patch didn't solve our problem
+			QCoreApplication::processEvents(QEventLoop::AllEvents);
+		#endif
 		scene()->update();
 	}
 
@@ -128,7 +130,6 @@ protected:
 	//! This method is called automatically by Qt.
 	virtual void drawBackground(QPainter* painter, const QRectF &rect);
 
-
 signals:
 	//! emitted when saveScreenShot is requested with saveScreenShot().
 	//! doScreenshot() does the actual work (it has to do it in the main
@@ -186,6 +187,7 @@ private:
 	float minfps;
 	//! The maximum desired frame rate in frame per second.
 	float maxfps;
+        bool doPaint;
 };
 
 
