@@ -93,6 +93,10 @@ namespace StelUtils
 	//! @param decimal output decimal second value
 	QString radToDmsStr(double angle, bool decimal=false, bool useD=false);
 
+	//! Convert a dms formatted string to an angle in radian
+	//! @param dsm The input string
+	double dmsStrToRad(const QString& s);
+
 	//! Obtains a Vec3f from a string.
 	//! @param s the string describing the Vector with the form "x,y,z"
 	//! @return The corresponding vector
