@@ -139,7 +139,7 @@ public slots:
 	bool getFlagLabels(void) const {return flagNames;}
 
 	//! Set whether selected constellation must be displayed alone
-	void setFlagIsolateSelected(bool s) { isolateSelected = s; }
+	void setFlagIsolateSelected(bool s);
 	//! Get whether selected constellation is displayed alone
 	bool getFlagIsolateSelected(void) const { return isolateSelected;}
 
@@ -212,6 +212,8 @@ private:
 	//! NOTE: this function should return a list of all, or may be deleted. Please
 	//! do not use until it exhibits the proper behaviour.
 	StelObject* getSelected(void) const;
+	//! Remove constellations from selected objects
+	void deselectConstellations(void);
 
 	std::vector<Constellation*> selected; // More than one can be selected at a time
 
